import pandas as pd
from pymongo import MongoClient
from pymongo.server_api import ServerApi


class MongoDb:

    def __init__(self, uri):
        """
        Initializes the MongoDb class with the provided URI.

        Args:
            uri (str): The URI for connecting to the MongoDB instance.
        """
        self.client = None
        self.db = None
        self.collection = None
        self.uri = uri

    def connect(self):
        """
        Establishes a connection to the MongoDB instance.
        """
        # Connect to MongoDB
        self.client = MongoClient(self.uri, server_api=ServerApi('1'))
        self.db = self.client['crypto']

    @staticmethod
    def df_to_json(df) -> list:
        """
        Converts a DataFrame to a list of JSON documents.

        Args:
            df (pd.DataFrame): The DataFrame to be converted.

        Returns:
            list: A list of JSON documents.
        """
        # Convert each row to a JSON document
        json_docs = []
        for index, row in df.iterrows():
            json_doc = row.to_dict()  # Convert row to dictionary
            json_docs.append(json_doc)
        return json_docs

    def insert_json(self, json_data, collection):
        """
        Inserts a JSON document into the specified collection.

        Args:
            json_data (dict): The JSON document to be inserted.
            collection (str): The name of the collection to insert into.
        """
        try:
            self.connect()
            self.collection = self.db[collection]
            # Insert data
            insert_result = self.collection.insert_one(json_data)
            print('Eklenen belge ID:', insert_result.inserted_id)
            self.client.close()
        except Exception as e:
            print("Insert_data Error: ", e)

    def insert_data(self, df, collection):
        """
        Inserts data from a DataFrame into the specified collection.

        Args:
            df (pd.DataFrame): The DataFrame containing the data to be inserted.
            collection (str): The name of the collection to insert into.
        """
        json_docs = self.df_to_json(df)

        # Insert documents
        for json_doc in json_docs:
            self.insert_json(json_doc, collection)
            print(json_doc)

    def get_data(self, collection, query=None):
        """
        Retrieves data from the specified collection.

        Args:
            collection (str): The name of the collection to retrieve data from.
            query (dict): The query to filter the data (optional).

        Returns:
            pd.DataFrame: A DataFrame containing the retrieved data.
        """
        self.connect()
        if query is None:
            # Retrieve data
            my_collection = self.db[collection]
            data = list(my_collection.find())
            df = pd.DataFrame(data)
            self.client.close()
            return df
        else:
            my_collection = self.db[collection]
            data = my_collection.find(query)
            df = pd.DataFrame(data)
            self.client.close()
            return df

    def check_collection_name(self, collection_name):
        """
        Checks if a collection with the specified name exists.

        Args:
            collection_name (str): The name of the collection to check.

        Returns:
            bool: True if the collection exists, False otherwise.
        """
        self.connect()

        # Check if collection exists
        if collection_name in self.db.list_collection_names():
            print(f"'{collection_name}' koleksiyonu mevcut.")
            return True
        else:
            print(f"'{collection_name}' koleksiyonu mevcut değil.")
            return False

        self.client.close()

    def create_collection(self, collection_name):
        """
        Creates a new collection with the specified name.

        Args:
            collection_name (str): The name of the new collection to create.
        """
        self.connect()

        # Create collection
        collection = self.db[collection_name]
        self.client.close()



from .mongodb import MongoDb
from .MySQLConnector import MySQLConnector


class dataconverter:
    def __init__(self, mongo_uri, **kwargs):
        """
        Initializes the Converter class.

        Args:
            mongo_uri (str): The URI for connecting to MongoDB.
            **kwargs: Keyword arguments containing MySQL connection details.
                      Expected keys: 'host', 'user', 'password', 'port', 'database'.
        """
        self.mongo_uri = mongo_uri

        self.host = kwargs['host']
        self.user = kwargs['user']
        self.password = kwargs['password']
        self.port = kwargs['port']
        self.database = kwargs['database']

        self.mysql_conn = MySQLConnector(self.host, self.user, self.password, self.port, self.database)
        self.mongo_conn = MongoDb(self.mongo_uri)

    def mysql_to_mongodb(self, table_name=None):
        """
        Migrates data from MySQL to MongoDB.

        Args:
            table_name (str, optional): The name of the table to migrate (default is None).
                                        If None, all tables will be migrated.

        Returns:
            None
        """

        if table_name is None:
            print("bütün db geçirelecek")
        else:
            df = self.mysql_conn.get_data(table_name)
            if self.mongo_conn.check_collection_name(table_name):
                self.mongo_conn.insert_data(df, table_name)
            else:  # Collection mevcut değil.
                self.mongo_conn.create_collection(table_name)
                self.mongo_conn.insert_data(df, table_name)

    def mongodb_to_mysql(self, collection_name=None):
        """
        Migrates data from MongoDB to MySQL.

        Args:
            collection_name (str, optional): The name of the collection to migrate (default is None).
                                             If None, all collections will be migrated.

        Returns:
            None
        """
        if collection_name is None:
            print("bütün db geçirelecek")
        else:
            df = self.mongo_conn.get_data(collection_name)
            if self.mysql_conn.check_table_name(collection_name):  # tablo varsa
                self.mysql_conn.insert_data(df, collection_name)
            else:  # tablo yoksa
                self.mysql_conn.create_table(table_name=collection_name, df=df)
                self.mysql_conn.insert_data(df, collection_name)



from .converter import dataconverter

# DW bilgieri
DATAWAREHOUSE_HOST = "172.16.5.16"
DATAWAREHOUSE_USER = "bi-admin"
DATAWAREHOUSE_PASSWORD = "Dk5Bnqgj3gEsRz2p"
port_number = 3308


uri = "mongodb+srv://mert:Mert123456!@cluster0.smovknl.mongodb.net/?retryWrites=true&w=majority"
mysql_bilgilerim = {'host': DATAWAREHOUSE_HOST, 'user': DATAWAREHOUSE_USER, 'password': DATAWAREHOUSE_PASSWORD, 'port': port_number, 'database': 'weg_hotel'}

my_converter = dataconverter(uri, **mysql_bilgilerim)
my_converter.mysql_to_mongodb('hotels_pro_b2b_test')

"""
my_converter = Converter(
    mongo_uri="mongodb+srv://admin:password!@cluster0.smovknl.mongodb.net/?retryWrites=true&w=majority",
    host="000.00.0.00",
    user="admin",
    password="password",
    port=8080,
    database="your_db_name"
)

# Migrate data from MySQL to MongoDB
my_converter.mysql_to_mongodb(table_name="test_table")

# Migrate data from MongoDB to MySQL
my_converter.mongodb_to_mysql(collection_name="my_collection")

"""


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.2.2'
DESCRIPTION = 'Data migration. MongoDB to MySQL - MySQL to MongoDB'
#LONG_DESCRIPTION = 'A package that allows to migrate your data.'

# Setting up
setup(
    name="db-converter",
    version=VERSION,
    author="dilsizyazilimci (Mert Celikan)",
    author_email="celikanmert@gmail.com",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[  # Gerekli bağımlılıklar
        'numpy',
        'pandas',
        'pymongo',
        'PyMySQL',
        'python-dateutil',
        'pytz',
        'six',
        'dnspython'
    ],
    keywords=['python', 'mongodb', 'mysql', 'database', 'MySQL to MongoDB', 'MongoDB to MySQL', 'data migration'],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


import pymysql
import datetime
import pandas as pd


class MySQLConnector:
    def __init__(self, host, user, password, port_number, database):
        """
        Initializes the MySQLConnector with connection details.

        Args:
            host (str): The host address of the MySQL server.
            user (str): The MySQL user.
            password (str): The password for the MySQL user.
            port_number (int): The port number for the MySQL server.
            database (str): The name of the database.
        """
        self.host = host
        self.user = user
        self.password = password
        self.port_number = port_number
        self.database = database
        self.connection = None

    def connect(self):
        """
        Establishes a connection to the MySQL database.
        """
        self.connection = pymysql.connect(
            host=self.host,
            user=self.user,
            password=self.password,
            port=self.port_number,
            database=self.database
        )

    def disconnect(self):
        """
        Closes the connection to the MySQL database.
        """
        if self.connection:
            self.connection.close()

    def get_data(self, table) -> pd.DataFrame:
        """
        Retrieves data from a specific table in the MySQL database and returns it as a DataFrame.

        Args:
            table (str): The name of the table to retrieve data from.

        Returns:
            pd.DataFrame: A DataFrame containing the retrieved data.
        """
        self.connect()
        cursor = self.connection.cursor()
        cursor.execute(f"SELECT * FROM {table}")
        # Sütun isimlerini al
        columns = [desc[0] for desc in cursor.description]
        result = cursor.fetchall()
        df = pd.DataFrame(result, columns=columns)

        for col in df.columns:
            if df[col].dtype == 'object' and isinstance(df[col].values[0], datetime.date):
                df[col] = df[col].astype(str)

        self.disconnect()
        return df

    def insert_data(self, df, table_name):
        """
        Inserts data from a DataFrame into a specified table in the MySQL database.

        Args:
            df (pd.DataFrame): The DataFrame containing the data to be inserted.
            table_name (str): The name of the table to insert data into.
        """
        print("SQL Conn kuruluyor...")

        self.connect()
        if '_id' in df.columns:
            df = df.drop('_id', axis=1)
        values = ','.join(['%s'] * len(df.columns))

        sql = f"INSERT INTO {table_name} ({','.join(df.columns)}) VALUES ({values})"
        # Verileri tuple olarak dönüştürme
        data = [tuple(row) for row in df.values]

        # INSERT sorgusunu çalıştırma
        with self.connection.cursor() as cursor:
            cursor.executemany(sql, data)
            self.connection.commit()

        # Close the cursor and connection
        cursor.close()
        self.disconnect()

    def check_table_name(self, table_name):
        """
        Checks if a table with the specified name exists in the MySQL database.

        Args:
            table_name (str): The name of the table to check.

        Returns:
            bool: True if the table exists, False otherwise.
        """
        self.connect()
        # Bağlantı üzerinden bir cursor oluştur
        cursor = self.connection.cursor()

        # SQL sorgusu ile tablonun var olup olmadığını kontrol et
        query = f"SHOW TABLES LIKE '{table_name}'"
        cursor.execute(query)

        # fetchone metodu ile sonuçları al
        result = cursor.fetchone()

        # Bağlantıyı kapat
        cursor.close()
        self.connection.close()

        # Sonuçları kontrol et
        if result:
            print(f"'{table_name}' tablosu mevcut.")
            return True
        else:
            print(f"'{table_name}' tablosu mevcut değil.")
            return False
        self.disconnect()

    def create_table(self, table_name, df):
        """
        Creates a table in the MySQL database based on the DataFrame schema.

        Args:
            table_name (str): The name of the table to create.
            df (pd.DataFrame): The DataFrame containing the schema of the table.
        """
        self.connect()

        if '_id' in df.columns:
            df = df.drop('_id', axis=1)

        # Veri tiplerini inceleyin
        data_types = df.dtypes

        # MySQL veri tiplerini belirle
        mysql_data_types = {
            'int64': 'INT',
            'float64': 'FLOAT',
            'object': 'VARCHAR(255)', # Örnek olarak VARCHAR(255) seçildi, ihtiyaca göre değiştirilebilir.
            'datetime64[ns]': 'DATETIME',
        }

        # Bağlantı üzerinden bir cursor oluştur
        cursor = self.connection.cursor()

        # Tablo adı ve sütunlar
        columns = ', '.join([f'{col} {mysql_data_types[str(data_types[col])]}'
                             for col in df.columns])

        # SQL sorgusu ile tabloyu oluştur
        create_table_query = f"CREATE TABLE {table_name} ({columns})"
        cursor.execute(create_table_query)

        cursor.close()
        self.disconnect()
